package com.cringe_studios.cringe_authenticator_library.impl;

import com.cringe_studios.cringe_authenticator_library.OTPAlgorithm;
import com.cringe_studios.cringe_authenticator_library.OTPException;
import com.cringe_studios.cringe_authenticator_library.OTPType;

//https://datatracker.ietf.org/doc/html/rfc6238
public class TOTP extends HOTP {

	public TOTP(String secret, OTPAlgorithm algorithm, int digits, long counter, long periodInSeconds, boolean checksum) {
		super(secret, algorithm, digits, counter, periodInSeconds, checksum);
		this.type = OTPType.TOTP;
	}

	@Override
	public long getNextDueTime() {
		// e.g. current time: 20 seconds & current period: 30 seconds -> next due time:
		// 30 seconds
		return (this.getCounter() + 1) * this.getPeriod();
	}

	@Override
	public String getPin() throws OTPException {
		return getPinAtTime(System.currentTimeMillis() / 1000);
	}

	@Override
	public long getCounter() {
		return getCounterAt(System.currentTimeMillis() / 1000);
	}

	private long getCounterAt(long unixSecond) {
		return (unixSecond - getTimeCorrection()) / this.getPeriod();
	}

	public String getPinAtTime(long unixSecond) throws OTPException {
		return getPinAtCounter(getCounterAt(unixSecond));
	}

	@Override
	public void incrementCounter() {}
}